#!/usr/bin/ruby

require 'csv'
require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
    class Game
        def initialize(name)
            @title = name
            @players = []
        end

        def add_player(player)
            @players << player
        end

        def load_players(from_file)
            CSV.foreach(from_file) do |row|
                add_player(Player.new(row[0], Integer(row[1])))
            end
        end

        def play(rounds)
            puts "There are #{@players.size} players in #{@title}:"
            puts @players
            treasures = TreasureTrove::TREASURES
            puts "There are #{treasures.size} treasures to be found:"
            treasures.each do |treasure|
                puts "A #{treasure.name} is worth #{treasure.points} points"
            end
            1.upto(rounds) do |round|
                break if block_given? and yield
                puts "\nRound #{round}"
                @players.each do |player|
                    GameTurn.take_turn(player)
                end
            end
        end

        def total_points
            @players.reduce(0) do |sum, player|
                sum + player.points
            end
        end

        def high_scores
            scores = "#{@title} High Scores:"
            @players.sort.each do |player|
                scores += "\n#{player.name.ljust(20, ".")} #{player.score}"
            end
            scores
        end

        def print_stats
            puts "#{@title} Statistics:"
            strong, whimpy = @players.partition { |player| player.strong? }
            strong_sorted_by_health = strong.sort { |a, b| b.health <=> a.health }
            whimpy_sorted_by_health = whimpy.sort { |a, b| b.health <=> a.health }

            puts "\n#{strong.size} strong players:"
            strong_sorted_by_health.each do |player|
                puts "#{player.name} (#{player.health})"
            end

            puts "\n#{whimpy.size} whimpy players:"
            whimpy_sorted_by_health.sort.each do |player|
                puts "#{player.name} (#{player.health})"
            end

            puts "\n" + high_scores
       
            @players.each do |player|
                puts "\n#{player.name}'s point totals:"
                player.each_found_treasure do |treasure|
                    puts "#{treasure.points} total #{treasure.name} points"
                end
                puts "#{player.points} grand total points"
            end

            puts "\n#{total_points} total points from treasures found"
        end

        def save_high_scores(to_file="high_scores.txt")
            File.open(to_file, "w") do |file|
                file.puts high_scores
            end
        end

    end
end
