#!/usr/bin/ruby

require_relative 'playable'

module StudioGame

    class Player
        include Playable

        attr_accessor :health
        attr_accessor :name

        def name=(new_name)
            @name = new_name.capitalize
        end

        def initialize(name, health=100)
            @name = name.capitalize
            @health = health
            @found_treasures = Hash.new(0)
        end

        def self.from_csv(line)
            name, health = line.split(',')
            Player.new(name, Integer(health))
        end

        def found_treasure(treasure)
            @found_treasures[treasure.name] += treasure.points
            puts "#{@name} found #{treasure.name} worth of #{treasure.points}"
            puts "#{@name}'s treasures: #{@found_treasures}"
        end

        def points 
            @found_treasures.values.reduce(0, :+)
        end

        def to_s
            "I'm #{@name} with health = #{@health}, points = #{points} and score = #{score}"
        end

        def score 
            @health + points
        end

        def <=>(player)
            player.score <=> score
        end

        def each_found_treasure
            @found_treasures.each do |name, points|
                yield(Treasure.new(name, points))
            end
        end

    end

end

if __FILE__ == $0
    player = StudioGame::Player.new("moe")
    puts player.name
    puts player.health
    player.w00t
    puts player.health
    player.blam
    puts player.health
end

